Package: qpgraph
Title: Estimation of Genetic and Molecular Regulatory Networks from
        High-Throughput Genomics Data
Version: 2.46.0
Authors@R: c(person("Robert", "Castelo", role=c("aut", "cre"), email="robert.castelo@upf.edu"),
             person("Alberto", "Roverato", role="aut", email="alberto.roverato@unibo.it"))
Description: Estimate gene and eQTL networks from high-throughput expression and genotyping assays.
License: GPL (>= 2)
Depends: R (>= 3.5)
Imports: methods, parallel, Matrix (>= 1.5-0), grid, annotate, graph
        (>= 1.45.1), Biobase, S4Vectors, BiocParallel, AnnotationDbi,
        IRanges, Seqinfo, GenomicRanges, GenomicFeatures, mvtnorm, qtl,
        Rgraphviz
Suggests: RUnit, BiocGenerics, BiocStyle, genefilter, org.EcK12.eg.db,
        rlecuyer, snow, Category, GOstats
LazyData: yes
URL: https://github.com/rcastelo/qpgraph
BugReports: https://github.com/rcastelo/qpgraph/issues
biocViews: Microarray, GeneExpression, Transcription, Pathways,
        NetworkInference, GraphAndNetwork, GeneRegulation, Genetics,
        GeneticVariability, SNP, Software
git_url: https://git.bioconductor.org/packages/qpgraph
git_branch: RELEASE_3_23
git_last_commit: 05e827a
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-29 00:02:59 UTC; biocbuild
Author: Robert Castelo [aut, cre],
  Alberto Roverato [aut]
Maintainer: Robert Castelo <robert.castelo@upf.edu>
