setGeneric("rgraphBAM", function(n, param, ...) standardGeneric("rgraphBAM"))
setGeneric("UGgmm", function(g, ...) standardGeneric("UGgmm"))
setGeneric("HMgmm", function(g, ...) standardGeneric("HMgmm"))
setGeneric("rUGgmm", function(n, g, ...) standardGeneric("rUGgmm"))
setGeneric("rHMgmm", function(n, g, ...) standardGeneric("rHMgmm"))
setGeneric("eQTLcross", function(map, genes, model, ...) standardGeneric("eQTLcross"))
setGeneric("addGenes", function(eQTLcross, n, ...) standardGeneric("addGenes"))
setGeneric("addeQTL", function(eQTLcross, ...) standardGeneric("addeQTL"))
setGeneric("addGeneAssociation", function(eQTLcross, ...) standardGeneric("addGeneAssociation"))
setGeneric("reQTLcross", function(n, network, ...) standardGeneric("reQTLcross"))
setGeneric("alleQTL", function(x, ...) standardGeneric("alleQTL"))
setGeneric("ciseQTL", function(x, cisr, ...) standardGeneric("ciseQTL"))
setGeneric("allGeneAssociations", function(x, ...) standardGeneric("allGeneAssociations"))
setGeneric("transeQTL", function(x, cisr, ...) standardGeneric("transeQTL"))
setGeneric("rcmvnorm", function(n, model, ...) standardGeneric("rcmvnorm"))

setGeneric("qpCItest", function(X, ...) standardGeneric("qpCItest"))
setGeneric("qpAllCItests", function(X, ...) standardGeneric("qpAllCItests"))
setGeneric("qpNrr", function(X, ...) standardGeneric("qpNrr"))
setGeneric("qpEdgeNrr", function(X, ...) standardGeneric("qpEdgeNrr"))
setGeneric("qpEdgeCor", function(X, ...) standardGeneric("qpEdgeCor"))
setGeneric("qpPathWeight", function(X, ...) standardGeneric("qpPathWeight"))
setGeneric("qpGraph", function(nrrMatrix, ...) standardGeneric("qpGraph"))

setGeneric("geneticMap", function(object) standardGeneric("geneticMap"))
setGeneric("physicalMap", function(object) standardGeneric("physicalMap"))
setGeneric("ggData", function(object) standardGeneric("ggData"))
setGeneric("markerNames", function(object) standardGeneric("markerNames"))
setGeneric("geneNames", function(object) standardGeneric("geneNames"))
setGeneric("graph", function(object) standardGeneric("graph"))
setGeneric("geneAnnotation", function(object) standardGeneric("geneAnnotation"))
setGeneric("eQTLnetworkEstimate", function(param, model, estimate, ...) standardGeneric("eQTLnetworkEstimate"))
setGeneric("varExplained", function(param, estimate, ...) standardGeneric("varExplained"))
setGeneric("resetCutoffs", function(object, ...) standardGeneric("resetCutoffs"))
